package com.dzeio.openhealth.connectors.samsunghealth

import android.app.Activity
import android.app.AlertDialog
import android.content.DialogInterface
import android.util.Log
import com.samsung.android.sdk.healthdata.*
import com.samsung.android.sdk.healthdata.HealthDataStore.ConnectionListener
import com.samsung.android.sdk.healthdata.HealthPermissionManager.*
import com.samsung.android.sdk.healthdata.HealthResultHolder.ResultListener
import java.lang.Boolean
import kotlin.Exception


class SamsungHealth(
    private val context: Activity
) {
    companion object {
        const val TAG = "SamsungHealthConnector"
    }
    private val store: HealthDataStore get() = _store!!
    private var _store: HealthDataStore? = null
    private val keySet: HashSet<PermissionKey> = HashSet()

    private val permissionListener: ResultListener<PermissionResult> =
        ResultListener<PermissionResult> { result ->
            Log.d(TAG, "Permission callback is received.")
            val resultMap: Map<PermissionKey, kotlin.Boolean> = result!!.resultMap

            if (resultMap.containsValue(Boolean.FALSE)) {
                // Requesting permission fails
                Log.d(TAG, "Requesting permission fails")

            } else {
                // Get the current step count and display it
                getStepCount()
            }
        }

    private val connectionListener: ConnectionListener = object : ConnectionListener {
        override fun onConnected() {
            Log.d(TAG, "Health data service is connected.")
            val pmsManager = HealthPermissionManager(store)
            try {
                val resultMap = pmsManager.isPermissionAcquired(keySet)

                if (resultMap.containsValue(Boolean.FALSE)) {
                    // Request the permission for reading step counts if it is not acquired
                    pmsManager.requestPermissions(keySet, context)
                        .setResultListener(permissionListener)
                } else {
                    // Get the current step count and display it
                    getStepCount()
                }
            } catch (e: Exception) {
                Log.e(TAG, e.javaClass.name + " - " + e.message)
                Log.e(TAG, "Permission setting fails.")
            }
        }

        override fun onConnectionFailed(error: HealthConnectionErrorResult) {
            Log.d(TAG, "Health data service is not available.")
            showConnectionFailureDialog(error)
        }

        override fun onDisconnected() {
            Log.d(TAG, "Health data service is disconnected.")
        }
    }

    init {
        //keySet = HashSet()
        keySet.add(PermissionKey(HealthConstants.Weight.HEALTH_DATA_TYPE, PermissionType.READ))

        _store = HealthDataStore(context, connectionListener)
    }

    fun test() {
        store.connectService()
    }

    private fun getStepCount() {
        val resolver = HealthDataResolver(store, null)
        val request = HealthDataResolver.ReadRequest.Builder()
            .setDataType(HealthConstants.Weight.HEALTH_DATA_TYPE)
            .setLocalTimeRange(HealthConstants.Weight.START_TIME, HealthConstants.Weight.TIME_OFFSET, 0, System.currentTimeMillis())
            .build()

        try {
            val res = resolver.read(request).await()

            res.use { res1 ->
                val iterator = res1.iterator()
                if (iterator.hasNext()) {
                    val data = iterator.next()
                    val value = data.getFloat(HealthConstants.Weight.WEIGHT)
                    val time = data.getLong(HealthConstants.Weight.CREATE_TIME)
                    Log.d(TAG, "Data received! $value $time")
                }
            }
        } catch (e : Exception) {
            Log.d(TAG, "Reading failed!")
        }
    }

    private fun showConnectionFailureDialog(error: HealthConnectionErrorResult) {
        val alert: AlertDialog.Builder = AlertDialog.Builder(context)
        var message = "Connection with Samsung Health is not available"
        if (error.hasResolution()) {
            message = when (error.errorCode) {
                HealthConnectionErrorResult.PLATFORM_NOT_INSTALLED -> "Please install Samsung Health"
                HealthConnectionErrorResult.OLD_VERSION_PLATFORM -> "Please upgrade Samsung Health"
                HealthConnectionErrorResult.PLATFORM_DISABLED -> "Please enable Samsung Health"
                HealthConnectionErrorResult.USER_AGREEMENT_NEEDED -> "Please agree with Samsung Health policy"
                else -> "Please make Samsung Health available"
            }
        }
        alert.setMessage(message)
        alert.setPositiveButton("OK", DialogInterface.OnClickListener { dialog, id ->
            if (error.hasResolution()) {
                error.resolve(context)
            }
        })
        if (error.hasResolution()) {
            alert.setNegativeButton("Cancel", null)
        }
        alert.show()
    }


    fun destroy() {
        store.disconnectService()
    }
}